var VbStats;
if (!VbStats) {
	VbStats = {};
}

VbStats.ChunksSwarmCdnViz = function(svg, channelManager, options) {
	this.downPdnLabel = "Downloaded from viblast PDN";
	this.downCdnLabel = "Downloaded from CDN";
	this.notDownloadedLabel = "Not downloaded yet";
	
	this.chunkSize = 20;
	this.swarmAttrs = {"fill": "#0fa8d3"};
	this.cdnAttrs = {"fill": "#f2a300"};
	this.notDownloadedAttrs = {"fill": "#a2a2a2"};
	this.margin = 5;
	this.numChunks = 25;
	this.abi = -1;
	this.played = -1;
	this.lastOffsetOffer = -1;
	this.showAdvancedFields = false;
	this.chunkUploadedStyle = 'num-uploads';
	
	if (options) {
		jQuery.extend(this, options);
	}
	
	Object.defineProperty(this, "showChunkIdx", {
		get : function(){ return this._showChunkIdx; },
        set : function(show){ 
        	this._showChunkIdx = show;
        	this.refreshShowChunkIdx();
        }
	});
	Object.defineProperty(this, "showChunkDuration", {
		get: function() { return this._showChunkDuration; },
		set: function(show) {
			this._showChunkDuration = show;
			this.refreshShowChunkDuration();
		}
	});
	
	//animation props
	this.animateDownloadedDuration = 1;
	this.moveProgressBarDuration = 1.5;
	this.moveMarginPercent = 0.5;
	this.showKey = true;
	
	this.moveMarginChunks = ~~(this.numChunks * this.moveMarginPercent);
	
	this.svg = svg;
	this.NS = "http://www.w3.org/2000/svg";
	
	var group = document.createElementNS(this.NS, "g");
	this.group = group;
	group.setAttribute("transform", "translate(0.5 0.5)");
	svg.append(group);
	
	this.chunks = [];
	this.offset = -1;
	
	var i = 0;
	for (i = 0; i<this.numChunks; i++) {
		var chunk = this.createChunk();
		this.setAttributes(chunk, this.notDownloadedAttrs);
		this.setAttributes(chunk, {translate: {x: i*this.chunkSize + i*this.margin, y:0}});
		group.appendChild(chunk);
		
		this.chunks.push(chunk);
	}
	
	if (this.showKey) {
		var key = document.createElementNS(this.NS, "g");
		group.appendChild(key);
		this.key = key;
		var text = null;
		
		var swarmChunk = this.createChunk();
		this.setAttributes(swarmChunk, this.swarmAttrs);
		this.setAttributes(swarmChunk, {translate: {x: 0, y: this.chunkSize + 3*this.margin}});
		key.appendChild(swarmChunk);
		text = this.createText(this.downPdnLabel);
		key.appendChild(text);
		this.setAttributes(text, {translate: {x: this.chunkSize + this.margin, y: 2*this.chunkSize + 3*this.margin - 5}, 'class':'cscv-desc-text'});
		
		
		var cdnChunk = this.createChunk();
		this.setAttributes(cdnChunk, this.cdnAttrs);
		this.setAttributes(cdnChunk, {translate: {x: 0, y: 2*this.chunkSize + 4*this.margin}});
		key.appendChild(cdnChunk);
		text = this.createText(this.downCdnLabel);
		key.appendChild(text);
		this.setAttributes(text, {translate: {x: this.chunkSize + this.margin, y: 3*this.chunkSize + 4*this.margin - 5}, 'class':'cscv-desc-text'});
		
		var ndChunk = this.createChunk();
		this.setAttributes(ndChunk, this.notDownloadedAttrs);
		this.setAttributes(ndChunk, {translate: {x: 0, y: 3*this.chunkSize + 5*this.margin}});
		key.appendChild(ndChunk);
		text = this.createText(this.notDownloadedLabel);
		key.appendChild(text);
		this.setAttributes(text, {translate: {x: this.chunkSize + this.margin, y: 4*this.chunkSize + 5*this.margin - 5}, 'class':'cscv-desc-text'});
	}
};

//VbStats.ChunksSwarmCdnViz.prototype = new Viblast.InternalsObserver();
//VbStats.ChunksSwarmCdnViz.prototype.constructor = VbStats.ChunksSwarmCdnViz;

VbStats.ChunksSwarmCdnViz.prototype.createChunk = function(chunkIdx) {
	var g = document.createElementNS(this.NS, "g");
	g.setAttribute("transform", "translate(0, 0)");
	var rect = document.createElementNS(this.NS, "rect");
	g.appendChild(rect);
	rect.setAttribute("x", 0);
	rect.setAttribute("y", 0);
	rect.setAttribute("width", this.chunkSize);
	rect.setAttribute("height", this.chunkSize);
	
	g.setAttribute("stroke", "black");
	g.setAttribute("stroke-width", 1);
	
	var idxText = this.createText(chunkIdx !== undefined ? (chunkIdx+'') : '');
	g.appendChild(idxText);
	g.idxText = idxText;
	idxText.setAttribute('x', this.chunkSize >> 1);
	idxText.setAttribute('y', this.chunkSize + 3);
	idxText.setAttribute('text-anchor', 'middle');
	idxText.setAttribute('dominant-baseline', 'hanging');
	idxText.setAttribute('font-size', 9);
	idxText.setAttribute('fill', 'black');
	idxText.setAttribute('stroke-width', 0);
	if (!this.showChunkIdx) idxText.setAttribute('visibility', 'collapse');
	
	var durationTextY = this.showChunkIdx ? this.chunkSize + 13 : this.chunkSize + 3;
	var durationText = this.createText('');
	g.appendChild(durationText);
	g.durationText = durationText;
	this.setAttributes(durationText, {'x': this.chunkSize >> 1,
				'y': durationTextY, 'text-anchor': 'middle', 'dominant-baseline': 'hanging',
				'font-size': 9, 'fill': 'black', 'stroke-width': 0});
	if (!this.showChunkDuration) durationText.setAttribute('visibility', 'collapse');
	
	return g;
};

VbStats.ChunksSwarmCdnViz.prototype.refreshShowChunkIdx = function() {
	var visibility = this.showChunkIdx ? 'visible' : 'collapse';
	var i = 0;
	for (i = 0; i<this.chunks.length; i++) {
		this.chunks[i].idxText.setAttribute('visibility', visibility);
	}
};

VbStats.ChunksSwarmCdnViz.prototype.refreshShowChunkDuration = function() {
	var durationTextY = this.showChunkIdx ? this.chunkSize + 13 : this.chunkSize + 3;
	var visibility = this.showChunkDuration ? 'visible' : 'collapse';
	var i = 0;
	for (i = 0; i<this.chunks.length; i++) {
		this.chunks[i].durationText.setAttribute('visibility', visibility);
		this.chunks[i].durationText.setAttribute('y', durationTextY);
	}
};

VbStats.ChunksSwarmCdnViz.prototype.onChunkDownloaded = function(chunkIdx, peer) {
	var chunk = this.getChunk(chunkIdx);
	if (!chunk) return;//some chunks that is not in our progress bar (too far behind or too far ahead).
	if (peer.isCDNPeer()) {
		this.setOrAnimateAttributes(this.animateDownloadedDuration, chunk, this.cdnAttrs);
	} else {
		this.setOrAnimateAttributes(this.animateDownloadedDuration, chunk, this.swarmAttrs);
	}
	
	this.refreshAdvancedFields();
};

VbStats.ChunksSwarmCdnViz.prototype.onChunkUploaded = function(chunkIdx) {
	var chunk = this.probeChunk(chunkIdx);
	if (!chunk) return;
	chunk.__uploaded = (chunk.__uploaded ? chunk.__uploaded : 0) + 1;

	if (this.chunkUploadedStyle === 'num-uploads') {
		if (chunk.__uploadedText) chunk.removeChild(chunk.__uploadedText);
		chunk.__uploadedText = this.createText(chunk.__uploaded + '');
		chunk.appendChild(chunk.__uploadedText);
		this.setAttributes(chunk.__uploadedText, {translate: {x: this.chunkSize-8, y:this.chunkSize-3},
			'fill': 'darkblue', 'stroke':'black', 'stroke-width':0});
	} else if (this.chunkUploadedStyle === 'change-border') {
		//animate only the furst time
		if (chunk.__uploaded === 1) {
			this.setOrAnimateAttributes(this.animateDownloadedDuration, chunk, {'stroke': 'MidnightBlue', 'stroke-width':3});
		} 
	}
};

VbStats.ChunksSwarmCdnViz.prototype.onHave = function(from, chunkIdx, chunkDuration) {
	if (!this.showAdvancedFields) return;
	this.refreshAdvancedFields();
	var chunk = this.probeChunk(chunkIdx);
	if (chunk) {
		if (!chunk.hasSwarm) {/* not beloging to a swarm peer */
			chunk.hasSwarm = !from.isCDNPeer(); 
			if (chunk.hasSwarm) {
				this.setAttributes(chunk, {'stroke': 'darkblue', 'stroke-dasharray' : '5,5'});
			} else {
				this.setAttributes(chunk, {'stroke': 'darkorange', 'stroke-dasharray' : '5,5'});
			}
		}
		
		this.setChunkDuration(chunk, chunkDuration);
	}
}

//private
VbStats.ChunksSwarmCdnViz.prototype.setChunkDuration = function(chunk, chunkDuration) {
	this.setText(chunk.durationText, ~~(chunkDuration/1000) + 's');
};

VbStats.ChunksSwarmCdnViz.prototype.setChunkIdx = function(chunk, chunkIdx) {
	this.setText(chunk.idxText, chunkIdx);
};

VbStats.ChunksSwarmCdnViz.prototype.refreshAdvancedFields = function() {
	if (this.showAdvancedFields && this.channelManager) {
		this.refreshAbiChunk(this.channelManager.abi);
		this.refreshPlayedChunk(this.channelManager.playedChunk);
		this.refreshLastOffsetOffer(this.channelManager.lastOffsetOffer);
	}
}

VbStats.ChunksSwarmCdnViz.prototype.refreshAbiChunk = function(newAbi) {
	var chunk = this.probeChunk(this.abi);
	if (chunk)
		this.setAttributes(chunk, {'stroke': 'black', 'stroke-width': 1});
	
	this.abi = newAbi;
	chunk = this.probeChunk(this.abi);
	if (chunk)
		this.setAttributes(chunk, {'stroke': 'darkmagenta', 'stroke-width': 2});
}

VbStats.ChunksSwarmCdnViz.prototype.refreshPlayedChunk = function(newPlayed) {
	var chunk = this.probeChunk(this.played);
	if (chunk)
		this.setAttributes(chunk, {'stroke': 'black', 'stroke-width': 1});
	
	this.played = newPlayed;
	chunk = this.probeChunk(this.played);
	if (chunk)
		this.setAttributes(chunk, {'stroke': 'darkgreen', 'stroke-width': 2});
}

VbStats.ChunksSwarmCdnViz.prototype.refreshLastOffsetOffer = function(newLastOffsetOffer) {
	var chunk = this.probeChunk(this.lastOffsetOffer);
	if (chunk)
		this.setAttributes(chunk, {'stroke': 'black', 'stroke-width': 1});
	
	this.lastOffsetOffer = newLastOffsetOffer;
	chunk = this.probeChunk(this.lastOffsetOffer);
	if (chunk)
		this.setAttributes(chunk, {'stroke': 'red', 'stroke-width': 2, 'stroke-dasharray' : '5,0'});
}

VbStats.ChunksSwarmCdnViz.prototype.probeChunk = function(chunkIdx) {
	//chunkIdx can be equal to this.offset if both are -1 (i.e. not yet initialized)
	if (chunkIdx < this.offset || this.offset < 0) return null;
	var idx = chunkIdx - this.offset;
	if (idx >= this.chunks.length-1) {
		return null;
	} else {
		return this.chunks[idx];
	}
};

VbStats.ChunksSwarmCdnViz.prototype.getChunk = function(chunkIdx) {
	if (chunkIdx < this.offset) return null;
	if (this.offset < 0) {
		this.setOffset(this.channelManager.lastOffsetOffer);
		return this.getChunk(chunkIdx);
	}
	var idx = chunkIdx - this.offset;
	if (idx >= this.chunks.length-1) {
		var newOffset = chunkIdx;
		if (this.offset >= 0) {
			newOffset -= this.moveMarginChunks;
		}
		this.setOffset(newOffset);
		return this.getChunk(chunkIdx);
	} else {
		return this.chunks[idx];
	}
};

VbStats.ChunksSwarmCdnViz.prototype.createText = function(title) {
	var text = document.createElementNS(this.NS, "text");
	var textStr = document.createTextNode(title);
	text.appendChild(textStr);
	text.setAttribute("x", 0);
	text.setAttribute("y", 0);
	
	return text;
};

VbStats.ChunksSwarmCdnViz.prototype.onUpdateWindow = function(newOffset, forced) {
	if (forced) {
		this.offset = newOffset;
		var i = 0;
		for (i = 0; i<this.chunks.length; i++) {
			//fade out old chunks
			this.animateAttributes(this.moveProgressBarDuration/2.0, this.chunks[i],
					{translate: {x: i*this.chunkSize + i*this.margin, y: 1.5*this.chunkSize}, 'fill-opacity':0, 'stroke-opacity':0},
					function(chunk) {chunk.parentNode.removeChild(chunk);}
			);
			
			//fade in new ones
			var chunk = this.createChunk(this.offset + i);
			this.setAttributes(chunk, this.notDownloadedAttrs);
			this.setAttributes(chunk, {translate: {x: i*this.chunkSize + i*this.margin, y:-1.5*this.chunkSize}});
			this.group.appendChild(chunk);
			this.chunks[i] = chunk;
			
			this.setAttributes(chunk, {'fill-opacity':0, 'stroke-opacity':0});
			this.animateAttributes(this.moveProgressBarDuration/2.0, chunk,
				{translate: {x: i*this.chunkSize + i*this.margin, y: 0}, 'fill-opacity':1, 'stroke-opacity':1});
		}
	} else if (this.offset < 0) {
		this.setOffset(newOffset);
	}
}

VbStats.ChunksSwarmCdnViz.prototype.setOffset = function(offset) {
	if (this.offset < 0) {
		this.offset = offset;
		//move the key down if required
		if (this.showChunkIdx && this.showChunkDuration) {
			this.key.setAttribute("transform", "translate(0, 10)");
		}
		//set chunk indices if required
		var i = 0;
		for (i = 0; i<this.chunks.length; i++) {
			this.setChunkIdx(this.chunks[i], offset+i);
		}
		return;
	}
	if (offset < this.offset) throw "Moving the offset backwards is not yet implemented";
	var diff = offset - this.offset;
	diff = diff > this.chunks.length ? this.chunks.length : diff;
	this.offset = offset;
	var i = 0;
	for (i = 0; i<diff; i++) {
		this.animateAttributes(this.moveProgressBarDuration, this.chunks[i],
		    {translate: {x: -this.chunkSize - (diff - i)*this.chunkSize - (diff - i)*this.margin, y:0}, 'fill-opacity':0, 'stroke-opacity':0},
			function(chunk) {chunk.parentNode.removeChild(chunk);}
		);
	}
	
	for (i = diff; i<this.chunks.length; i++) {
		var newI = i-diff;
		
		this.animateAttributes(this.moveProgressBarDuration, this.chunks[i],
				{translate: {x: newI *this.chunkSize + newI*this.margin, y:0}});
		this.chunks[i-diff] = this.chunks[i];
	}
	
	var idx = 0;
	for (i = this.chunks.length - diff; i<this.chunks.length; i++) {
		var chunk = this.createChunk(offset + i);
		this.setAttributes(chunk, this.notDownloadedAttrs);
		this.group.appendChild(chunk);
		var width = this.numChunks*this.chunkSize + this.numChunks*this.margin + this.chunkSize;
		this.setAttributes(chunk, {translate: {x: width + idx*this.chunkSize + idx*this.margin, y:0}});
		
		this.setAttributes(chunk, {'fill-opacity':0, 'stroke-opacity':0});
		this.animateAttributes(this.moveProgressBarDuration, chunk,
			{translate: {x: i*this.chunkSize + i*this.margin, y:0}, 'fill-opacity':1, 'stroke-opacity':1});
		
		this.chunks[i] = chunk;
		idx++;
	}
};

VbStats.ChunksSwarmCdnViz.prototype.setOrAnimateAttributes = function(animateDur, svgObj, attrs) {
	if (animateDur > 0) {
		this.animateAttributes(animateDur, svgObj, attrs);
	} else {
		this.setAttributes(svgObj, attrs);
	}
};

VbStats.ChunksSwarmCdnViz.prototype.animateAttributes = function(dur, svgObj, props, onend) {
	var prop;
	var anims = [];
	for (prop in props) {
		var theProp = props[prop];
		if (prop !== 'translate') {
			var animate = document.createElementNS(this.NS, "animate");
			animate.setAttribute("attributeName", prop);
			animate.setAttribute("to", theProp);
			animate.setAttribute("dur", dur + "s");
			animate.setAttribute("repeatCount", 1);
			animate.setAttribute("fill", "freeze");
			animate.setAttribute("begin", this.svg[0].getCurrentTime());
			svgObj.appendChild(animate);
			anims.push(animate);
		} else {
			var animate = document.createElementNS(this.NS, "animateTransform");
			animate.setAttribute("attributeName", "transform"); 
			animate.setAttribute("type", prop);
			animate.setAttribute("from", svgObj.__x + ', ' + svgObj.__y);
			animate.setAttribute("to", theProp.x + ', ' + theProp.y);
			animate.setAttribute("dur", dur + "s");
			animate.setAttribute("repeatCount", 1);
			animate.setAttribute("fill", "freeze");
			animate.setAttribute("begin", this.svg[0].getCurrentTime());
			svgObj.appendChild(animate);
			anims.push(animate);
		}
	}
	//workaround for https://code.google.com/p/chromium/issues/detail?id=116595
	var thiz = this;
	window.setTimeout(function() {
			var i = 0;
			for (i = 0; i<anims.length; i++) {
				anims[i].parentNode.removeChild(anims[i]);
			}
			thiz.setAttributes(svgObj, props);
			if (onend) {
				onend(svgObj);
			}
	}, dur*1000);
};

VbStats.ChunksSwarmCdnViz.prototype.setAttributes = function(svgObj, props) {
	var prop;
	for (prop in props) {
		var theProp = props[prop];
		if (prop !== 'translate') { 
			svgObj.setAttribute(prop, theProp);
		} else {
			svgObj.setAttribute('transform', 'translate(' + theProp.x + ',' + theProp.y + ')');
			svgObj.__x = theProp.x;
			svgObj.__y = theProp.y;
		}
	}
};

VbStats.ChunksSwarmCdnViz.prototype.setText = function(textNode, text) {
	if (textNode.childNodes.length > 0) textNode.removeChild(textNode.childNodes[0]);
	var textStr = document.createTextNode(text);
	textNode.appendChild(textStr);
};
var Viblast = window['Viblast'];
if (!Viblast) Viblast = {};

Viblast.MultipleObserver = function(observers) {
	this.observers = observers;
}

Viblast.MultipleObserver.prototype.onInit = function() {
	this.call('onInit', arguments);
};

Viblast.MultipleObserver.prototype.onPeerStarted = function() {
	this.call('onPeerStarted', arguments);
};

Viblast.MultipleObserver.prototype.onPeerStopped = function() {
	this.call('onPeerStopped', arguments);
};

Viblast.MultipleObserver.prototype.refreshPeers = function() {
	this.call('refreshPeers', arguments);
};

Viblast.MultipleObserver.prototype.refreshBandwith = function() {
	this.call('refreshBandwith', arguments);
};

Viblast.MultipleObserver.prototype.refreshPeers = function() {
	this.call('refreshPeers', arguments);
};

Viblast.MultipleObserver.prototype.onChunkDownloaded = function() {
	this.call('onChunkDownloaded', arguments);
};

Viblast.MultipleObserver.prototype.onUpdateWindow = function() {
	this.call('onUpdateWindow', arguments);
};

Viblast.MultipleObserver.prototype.onHave = function() {
	this.call('onHave', arguments);
};

Viblast.MultipleObserver.prototype.onChunkUploaded = function() {
	this.call('onChunkUploaded', arguments);
};

Viblast.MultipleObserver.prototype.onPeerBlacklisted = function(peer) {
	this.call('onPeerBlacklisted', arguments);
};

Viblast.MultipleObserver.prototype.onCommJoined = function(peer) {
	this.call('onCommJoined', arguments);
};

Viblast.MultipleObserver.prototype.onPlayer = function() {
	this.call('onPlayer', arguments);
};

Viblast.MultipleObserver.prototype.onQualityChange = function() {
	this.call('onQualityChange', arguments);
};

Viblast.MultipleObserver.prototype.onAvailableVariantsUpdate = function() {
	this.call('onAvailableVariantsUpdate', arguments);
};

Viblast.MultipleObserver.prototype.call = function(event, args) {
	var i = 0;
	var observer = null;
	for (i = 0; i<this.observers.length; i++) {
		observer = this.observers[i];
		if (observer[event]) {
			observer.channelManager = this.channelManager;
			observer[event].apply(observer, args);
		}
	}
};
var Viblast = window['Viblast'];
if (!Viblast) Viblast = {};

Viblast.StatsForNerds = function(playerContainer) {
	this.playerContainer = playerContainer;
};

Viblast.StatsForNerds.prototype.fmtSize = function(sizeInBytes, precision) {
	if (sizeInBytes < 1024) {
		return [sizeInBytes.toFixed(precision), "B"];
	} else if (sizeInBytes < 1024*1024) {
		return [(sizeInBytes/1024).toFixed(precision), "KB"];
	} else if (sizeInBytes < 1024*1024*1024) {
		return [(sizeInBytes/(1024*1024)).toFixed(precision), "MB"];
	} else {
		return [(sizeInBytes/(1024*1024*1024)).toFixed(precision), "GB"];
	}
};

Viblast.StatsForNerds.prototype.fmtSpeed = function(size, precision) {
	size *= 8;

	if (size < 1024) {
		return [size.toFixed(precision), "bps"];
	} else if (size < 1024*1024) {
		return [(size/1024).toFixed(precision), "Kbps"];
	} else if (size < 1024*1024*1024) {
		return [(size/(1024*1024)).toFixed(precision), "Mbps"];
	} else {
		return [(size/(1024*1024*1024)).toFixed(precision), "Gbps"];
	}
};

Viblast.StatsForNerds.prototype.fmtPercent = function(a, b) {
	return (b ? (a/b)*100 : 0).toFixed(2);
};

Viblast.StatsForNerds.prototype.fmtRation = function(a, b) {
	return (b ? (a/b) : 0).toFixed(2);
};

Viblast.StatsForNerds.prototype.createRow = function(container, label) {
	var tr = document.createElement('tr');
	container.appendChild(tr);
	var td = document.createElement('td');
	td.style['padding-right'] = '10px';
	tr.appendChild(td);
	td.innerHTML = label;
	td = document.createElement('td');
	tr.appendChild(td);
	return td;
};

Viblast.StatsForNerds.prototype.injectCss = function() {
	var sheet = document.createElement('style');
	sheet.innerHTML = ".sfn { font: 14px/1.5 sans-serif; position: absolute; top: 10px; left: 10px; background: rgba(27, 27, 27, 0.9); color: #fff; z-index: 960; min-width:600px; }\
			.cmenu {\
				font-size: 12px;\
				font-family: arial, sans-serif;\
				padding: 0;\
				margin: 0;\
				-moz-user-select: none;\
				-ms-user-select: none;\
				-webkit-user-select: none;\
				-webkit-user-drag: none;\
				outline: none;\
				position: absolute;\
				border: 1px solid #ccc;\
				z-index: 2147483647;\
				overflow: auto;\
				background: #fff;\
				border-radius: 2px;\
			}\
			\
			.cmenu ul {\
				padding: 0;\
				margin: 0;\
			}\
			.cmenu li {\
				list-style: none;\
			}\
			\
			.cmenu span {\
				display: block;\
				margin: 0; \
				padding: 0 25px;\
				color: #333;\
				font-size: 13px;\
				text-decoration: none;\
				white-space: nowrap;\
				word-wrap: normal;\
				line-height: 25px;\
				cursor: pointer;\
			}\
			\
			.cmenu span:hover {\
				background: #333; \
				color:#fff;\
			}\
			.sfn canvas {\
				opacity: 0.5;\
			}\
			.sfn table, .sfn .flot-y-axis, .sfn .legend table td { color: #fff; }\
			.sfn .legend table {\
				border: none;\
			}\
			.sfn .legend .legendColorBox {\
				display: none;\
			}\
			.issueReport { position: absolute; top: 0; left: 0; background-color: rgb( 0, 0, 0 ); color: #fff; }\
			.issueReport p { margin: 0; padding: 0; }\
			.issueReport, .issueReport input, .issueReport textarea { font: 14px/1.5 sans-serif; }\
			.issueReport h3 { background-color: #333; margin: 0; padding: 5px; }\
			.issueReport .close { position: absolute; right: 5px; top: 5px; cursor: pointer; }\
			.issueReport .content { margin: 10px; }\
			.issueReport input, textarea { width: 100%; box-sizing: border-box; padding: 5px; }\
			.issueReport .content > input { float: right; width: 100px; margin: 10px; }"
			;
	document.head.insertBefore(sheet, document.head.firstChild);
};

Viblast.StatsForNerds.prototype.BuildGraphs = function( ) {
	function GetGraphContainer( name ) {
		var result = document.createElement('div');
		result.setAttribute( 'style', 'width: 300px; height:100px; position: relative; display: inline-block;');
		return result;
	}
	function BuildPDNCDNGraph( ) {
		function percentFormatter( val, axis ) {
			return val.toFixed(axis.tickDecimals) + "%";
		}
		BuildPDNCDNGraph.ClearGraph = function( ) {
			data.length = 0;

			for( var i = 0; i < 100; ++i )
				data.push( [i, 0] );

			if( plot ) {
				plot.setData(series);
				plot.draw();
			}
		}

		var graphHTML = GetGraphContainer( );

		var series = [{
			color: 'steelblue',
			data: [],
			label: "Traffic saved",
		}], data = series[0].data;

		BuildPDNCDNGraph.ClearGraph( );

		var plot = $.plot( graphHTML, series, {
			lines: { fill: true, fillColor: 'steelblue' },
			grid: { backgroundColor: 'darkorange', borderWidth: 0, labelMargin: 30 },
			xaxis: { show: false },
			yaxis: { min: 0, max: 100, labelWidth: 5, reserveSpace: true, font: { color: 'inherit' }, tickFormatter: percentFormatter, },
			legend: { show: true, position: "nw", }
		});

		BuildPDNCDNGraph.UpdateGraph = function( ) {
			var ratio = this.pdnDownloadInterval / ( this.pdnDownloadInterval + this.cdnDownloadInterval ) * 100;
			if( isNaN(ratio) )
				ratio = 0;

			data.shift();
			data.push([data.length, ratio]);

			for( var i = 0; i < data.length; ++i )
				data[i][0] = i;

			plot.setData(series);
			plot.draw();
		};

		return graphHTML;
	}

	function BuildUpDownRatio( ) {
		BuildUpDownRatio.ClearGraph = function( ) {
			data.length = 0;

			for( var i = 0; i < 100; ++i )
				data.push( [i, 0] );

			if( plot ) {
				plot.setData(series);
				plot.draw();
			}
		}

		var graphHTML = GetGraphContainer( );

		var series = [{
			color: 'steelblue',
			data: [],
			label: "Upload/Download ratio",
		}], data = series[0].data;

		BuildUpDownRatio.ClearGraph( );

		var plot = $.plot( graphHTML, series, {
			lines: { fill: true, fillColor: 'lightsteelblue' },
			grid: { borderWidth: 0, labelMargin: 30 },
			xaxis: { show: false },
			yaxis: { min: 0, labelWidth: 5, reserveSpace: true, font: { color: 'inherit' } },
			legend: { show: true, position: "nw", backgroundOpacity: 0, }
		});

		BuildUpDownRatio.UpdateGraph = function( ) {
			var ratio = this.pdnUploadInterval / ( this.cdnDownloadInterval + this.pdnDownloadInterval );
			if( isNaN(ratio) )
				ratio = 0;

			data.shift();
			data.push([data.length, ratio]);

			for( var i = 0; i < data.length; ++i )
				data[i][0] = i;

			plot.setData(series);
			plot.setupGrid( );
			plot.draw();
		}

		return graphHTML;
	}

	Viblast.StatsForNerds.prototype.UpdateGraphs = function( ) {
		BuildPDNCDNGraph.UpdateGraph.call( this );
		BuildUpDownRatio.UpdateGraph.call( this );
	}

	Viblast.StatsForNerds.prototype.ClearGraphs = function( ) {
		BuildPDNCDNGraph.ClearGraph.call( this );
		BuildUpDownRatio.ClearGraph.call( this );
	}

	var result = document.createElement('div');
	result.appendChild( BuildPDNCDNGraph( ) );
	result.appendChild( BuildUpDownRatio( ) );

	return result;
};

Viblast.StatsForNerds.prototype.UpdateGraphs = function( ) {

};

Viblast.StatsForNerds.prototype.onInit = function() {
	this.injectCss();
	this.menu = $('<div class="cmenu"></div>');
	var ul = $('<ul></ul>');
	this.menu.append(ul);
	
	//pause
	this.pauseLabel = document.createElement('span');
	var pauseItem = document.createElement('li');
	pauseItem.appendChild(this.pauseLabel);
	ul.append(pauseItem);
	pauseItem.onclick = this.togglePause.bind(this);
	
	//mute
	this.muteLabel = $('<span>Mute</span>')[0];
	var muteItem = $('<li></li>');
	muteItem.append(this.muteLabel);
	ul.append(muteItem);
	muteItem.click(this.toggleMute.bind(this));
	
	//hide controls
	this.controlsLabel = document.createElement('span');
	var controlsItem = document.createElement('li');
	controlsItem.appendChild(this.controlsLabel);
	ul.append(controlsItem);
	controlsItem.onclick = this.toggleControls.bind(this);
	
	//stats for nerds
	var statsNerdsitem = $('<li><span>Stats For Nerds</span></li>');
	statsNerdsitem.click(this.toggleStatsForNerds.bind(this));
	ul.append(statsNerdsitem);
	
	// Report Playback Issue
	var reportIssue = $('<li><span>Report Playback Issue</span></li>');
	reportIssue.click(this.toggleReportIssue.bind(this));
	ul.append(reportIssue);
	
	//sfn content
	var sfn = $('<div>').addClass( 'sfn' );

	//the x button
	var xButton = document.createElement('span');
	xButton.innerHTML = '[X]';
	xButton.setAttribute('style', "position:absolute; top: 5px; right: 5px; cursor: pointer;");
	xButton.onclick = this.hideStatsForNerds.bind(this);
	sfn.append($(xButton));
	
	var content = document.createElement('table');
	//num peers
	this.numPeersContainer = this.createRow(content, 'Peers:');
	this.setNumPeers(0);
	//total downloaded
	this.downloadHtml = this.createRow(content, 'Downloaded <span style="color: steelblue">PDN</span>/<span style="color: darkorange">CDN</span>/<span style="color: #2ca02c">Total</span>:');
	this.savings = this.createRow(content, 'Savings:');
	this.pdnUploadHtml = this.createRow(content, 'Uploaded to PDN:');
	this.speedsHTML = this.createRow(content, 'Speed <span style="color: steelblue">PDN</span>/<span style="color: darkorange">CDN</span>/<span style="color: darkmagenta">Upload</span>:');
	this.refreshBandwith();
	this.blacklistedHtml = this.createRow(content, 'Blacklisted:');
	this.permanentlyBlacklistedHtml = this.createRow(content, 'Permanently Blacklisted:');
	this.onPeerBlacklisted();
	
	sfn.append($(content));

	var graph = this.BuildGraphs( );
	sfn.append($(graph));

	this.sfn = sfn[0];
	this.sfn.style.display = 'none';

	// Issue Report content
	this.issueReport = document.createElement('form');
	this.issueReport.className = "issueReport";

	this.issueReport.innerHTML = '\
			<h3>Report Playback Issue</h3>\
			<span class="close">[X]</span>\
			<div class="content">\
				<p><label for="email">Email (optional):</label></p>\
				<p><input type="text" name="email" value=""></p>\
				<p><label for="feedback">Feedback:</label></p>\
				<textarea name="feedback" rows="5" cols="50"></textarea>\
				<input type="submit" value="Report issue">\
				<input type="reset" value="Cancel">\
			</div>';

	this.hideReportIssue( );
	this.issueReport.getElementsByClassName('close')[0].addEventListener('click', this.hideReportIssue.bind(this));
	this.issueReport.addEventListener( 'submit', this.sendReportIssue.bind(this) );
	this.issueReport.addEventListener( 'reset', this.hideReportIssue.bind(this) );
	var that = this;
	document.addEventListener( 'keyup', function( e ) {
		if( e.keyCode == 27 )
			that.hideReportIssue();
	});
};

Viblast.StatsForNerds.prototype.startRefreshing = function() {
	this.timerID = window.setInterval(this.refreshBandwith.bind(this), 1000);
	this.averagizeInit( );
	this.ClearGraphs( );
}

Viblast.StatsForNerds.prototype.stopRefreshing = function() {
	if( this.timerID ) {
		window.clearInterval(this.timerID);
		this.timerID = null;
	}
}

Viblast.StatsForNerds.prototype.toggleMute = function() {
	this.video.muted = !this.video.muted;
	this.refreshMuteLabel();
};

Viblast.StatsForNerds.prototype.refreshMuteLabel = function() {
	this.muteLabel.innerHTML = this.video.muted ? 'Unmute' : 'Mute';
};

Viblast.StatsForNerds.prototype.togglePause = function() {
	if (this.video.paused) this.video.play();
	else this.video.pause();
	this.refreshPauseLabel();
};

Viblast.StatsForNerds.prototype.refreshPauseLabel = function() {
	this.pauseLabel.innerHTML = this.video.paused ? 'Play' : 'Pause';
};

Viblast.StatsForNerds.prototype.toggleControls = function() {
	this.video.controls = !this.video.controls;
	this.refreshControlsLabel();
};

Viblast.StatsForNerds.prototype.refreshControlsLabel = function() {
	this.controlsLabel.innerHTML = this.video.controls ? 'Hide Controls' : 'Show Controls';
};

function mouseX(evt) {
	if (evt.pageX) {
		return evt.pageX;
	} else if (evt.clientX) {
	   return evt.clientX + (document.documentElement.scrollLeft ?
		   document.documentElement.scrollLeft :
		   document.body.scrollLeft);
	} else {
		return null;
	}
}

function mouseY(evt) {
	if (evt.pageY) {
		return evt.pageY;
	} else if (evt.clientY) {
	   return evt.clientY + (document.documentElement.scrollTop ?
	   document.documentElement.scrollTop :
	   document.body.scrollTop);
	} else {
		return null;
	}
}

Viblast.StatsForNerds.prototype.onPlayer = function() {
	var container = document.getElementById('player-container');
	var videoTags = container.getElementsByTagName('video');
	this.isHTML5Player  = videoTags.length > 0;
	if (this.isHTML5Player) {
		var video = videoTags[0];
		this.video = video;
		var thiz = this;
		video.addEventListener('contextmenu', function(event) {
			var docWidth = $(document).width();
			var docHeight = $(document).height();
			var htmlMenu = thiz.menu[0];
			$("body").append(thiz.menu);

			var left = mouseX(event);
			if ((left + thiz.menu.width()) > docWidth) {
				left -= thiz.menu.width();
			}
			thiz.menu.css('left', left);

			var top = mouseY(event);
			if ((top + thiz.menu.height()) > docHeight) {
				top -= thiz.menu.height();
			}
			thiz.menu.css('top', top);

			event.preventDefault();
		});
		document.addEventListener('click', function() {
			thiz.menu.detach();
		});
		window.addEventListener('blur', function() {
			thiz.menu.detach();
		});
		video.addEventListener('volumechange', this.refreshMuteLabel.bind(this));
		video.addEventListener('pause', this.refreshPauseLabel.bind(this));
		video.addEventListener('play', this.refreshPauseLabel.bind(this));
		this.refreshControlsLabel(); //not controls changed event
		this.refreshMuteLabel();

		$(video.parentNode).append(this.sfn);
		$(video.parentNode).append(this.issueReport)

		if (this.statsShown) {
			this.showStatsForNerds( );
		}
	}
};

Viblast.StatsForNerds.prototype.showStatsForNerds = function() {
	if (this.sfn) {
		this.sfn.style.display = 'initial';
		this.startRefreshing( );
	}

	this.statsShown = true;
};

Viblast.StatsForNerds.prototype.hideStatsForNerds = function() {
	this.sfn.style.display = 'none';
	this.stopRefreshing( );
	this.statsShown = false;
};

Viblast.StatsForNerds.prototype.toggleStatsForNerds = function( ) {
	if( this.statsShown )
		this.hideStatsForNerds( );
	else
		this.showStatsForNerds( );
};

Viblast.StatsForNerds.prototype.onPeerStarted = function() {
	this.setNumPeers(this.numPeers + 1);
};

Viblast.StatsForNerds.prototype.onPeerStopped = function() {
	this.setNumPeers(this.numPeers - 1);
}

Viblast.StatsForNerds.prototype.onPeerBlacklisted = function() {
	this.blacklistedHtml.innerHTML = this.channelManager.blacklisted;
	this.permanentlyBlacklistedHtml.innerHTML = this.channelManager.permanentlyBlacklisted;
}

Viblast.StatsForNerds.prototype.setNumPeers = function(num) {
	this.numPeers = num;
	this.numPeersContainer.innerHTML = num + "";
};

Viblast.StatsForNerds.prototype.refreshBandwith = function() {
	this.setDownload(this.channelManager.totalDownloadedFromCdn, this.channelManager.totalDownloadedFromSwarm);
	this.setSavings(this.channelManager.totalDownloadedFromSwarm);
	this.setPdnUpload(this.channelManager.totalUploadedToSwarm);
	this.setSpeeds( );

	this.averagize( );
	this.UpdateGraphs( );
};

Viblast.StatsForNerds.prototype.averagizeInit = function(hisoty_length) {
	var cdnDownloadHistory = [],
		pdnDownloadHistory = [],
		pdnUploadHistory = [],
		cdnDownloadTotal = 0,
		pdnDownloadTotal = 0,
		pdnUploadTotal = 0,
		cdnDownloadCurrent,
		pdnDownloadCurrent,
		pdnUploadCurrent,
		i;

	this.cdnDownloadInterval = 0;
	this.pdnDownloadInterval = 0;
	this.pdnUploadInterval = 0;

	this.cdnDownloadSmartInterval = 0;
	this.pdnDownloadSmartInterval = 0;
	this.pdnUploadSmartInterval = 0;

	hisoty_length = hisoty_length || 30;
	this.averagizeIntervalLength = hisoty_length;

	for( i = 0; i < hisoty_length; ++i ) {
		cdnDownloadHistory.push( 0 );
		pdnDownloadHistory.push( 0 );
		pdnUploadHistory.push( 0 );
	}

	Viblast.StatsForNerds.prototype.averagize = function() {
		cdnDownloadCurrent = this.channelManager.totalDownloadedFromCdn - cdnDownloadTotal;
		pdnDownloadCurrent = this.channelManager.totalDownloadedFromSwarm - pdnDownloadTotal;
		pdnUploadCurrent = this.channelManager.totalUploadedToSwarm - pdnUploadTotal;

		cdnDownloadTotal += cdnDownloadCurrent;
		pdnDownloadTotal += pdnDownloadCurrent;
		pdnUploadTotal += pdnUploadCurrent;

		this.cdnDownloadSmartInterval += hisoty_length*cdnDownloadCurrent - this.cdnDownloadInterval;
		this.cdnDownloadInterval -= cdnDownloadHistory.shift();
		cdnDownloadHistory.push(cdnDownloadCurrent);
		this.cdnDownloadInterval += cdnDownloadCurrent;

		this.pdnDownloadSmartInterval += hisoty_length*pdnDownloadCurrent - this.pdnDownloadInterval;
		this.pdnDownloadInterval -= pdnDownloadHistory.shift();
		pdnDownloadHistory.push(pdnDownloadCurrent);
		this.pdnDownloadInterval += pdnDownloadCurrent;

		this.pdnUploadSmartInterval += hisoty_length*pdnUploadCurrent - this.pdnUploadInterval;
		this.pdnUploadInterval -= pdnUploadHistory.shift();
		pdnUploadHistory.push(pdnUploadCurrent);
		this.pdnUploadInterval += pdnUploadCurrent;
	};
};

Viblast.StatsForNerds.prototype.averagize = function( ) {

};

Viblast.StatsForNerds.prototype.setDownload = function(cdnBytes, pdnBytes) {
	this.totalPdn = pdnBytes;
	this.totalCdn = cdnBytes;
	this.totalDownloaded = pdnBytes + cdnBytes;
	var totalFmt = this.fmtSize(this.totalDownloaded, 2);
	var pdnFmt = this.fmtSize(pdnBytes, 2);
	var cdnFmt = this.fmtSize(cdnBytes, 2);
	this.downloadHtml.innerHTML = '<span style="color: steelblue">' + pdnFmt[0] + ' ' + pdnFmt[1] + '</span> / ' +
									'<span style="color: darkorange">' + cdnFmt[0] + ' ' + cdnFmt[1] + '</span> / ' +
									'<span style="color: #2ca02c">' + totalFmt[0] + ' ' + totalFmt[1] + '</span>';
};

Viblast.StatsForNerds.prototype.setSavings = function(bytes) {
	this.pdnDownload = bytes;
	this.savings.innerHTML = this.fmtPercent(this.pdnDownload, this.totalDownloaded) + ' %';
};

Viblast.StatsForNerds.prototype.setSpeeds = function() {
	var smartLength = this.averagizeIntervalLength * ( this.averagizeIntervalLength + 1 ) / 2;
	var downCdnSpeed = this.cdnDownloadSmartInterval / smartLength;
	var downPdnSpeed = this.pdnDownloadSmartInterval / smartLength;
	var upSpeed = this.pdnUploadSmartInterval / smartLength;

	var downloadCdnFmt = this.fmtSpeed(downCdnSpeed, 2);
	var downloadPdnFmt = this.fmtSpeed(downPdnSpeed, 2);
	var uploadFmt = this.fmtSpeed(upSpeed, 2);
	this.speedsHTML.innerHTML = '<span style="color: steelblue">' + downloadPdnFmt[0] + ' ' + downloadPdnFmt[1] + '</span> / ' +
									'<span style="color: darkorange">' + downloadCdnFmt[0] + ' ' + downloadCdnFmt[1] + '</span> / ' +
									'<span style="color: darkmagenta">' + uploadFmt[0] + ' ' + uploadFmt[1] + '</span>';
};

Viblast.StatsForNerds.prototype.setPdnUpload = function(bytes) {
	this.pdnUplaoded = bytes;
	var fmt = this.fmtSize(bytes, 2);
	this.pdnUploadHtml.innerHTML = fmt[0] + ' ' + fmt[1];
};


Viblast.StatsForNerds.prototype.showReportIssue = function() {
	if (this.video)
		this.issueReport.style.display = 'initial';

	this.reportIssueShown = true;
};

Viblast.StatsForNerds.prototype.hideReportIssue = function() {
	this.issueReport.style.display = 'none';
	this.reportIssueShown = false;
};

Viblast.StatsForNerds.prototype.toggleReportIssue = function() {
	if( this.reportIssueShown )
		this.hideReportIssue( );
	else
		this.showReportIssue( );
};

Viblast.StatsForNerds.prototype.sendReportIssue = function( e ) {
	e.preventDefault( );

	var email = this.issueReport.elements.namedItem('email').value;
	var feedback = this.issueReport.elements.namedItem('feedback').value;

	$.ajax({
		type: "POST",
		url: "http://localhost:8000/rest/feedback/",
		contentType: "application/json",
		data: JSON.stringify({ email: email, feedback: feedback, vb_version: Viblast.version() }),
	});
};
